@@ -89,7 +89,7 @@ mod or_chip {
         pub fn new(state: &mut dyn State) -> Chip {
             let legs = Self::new_priv();
             let chip = Chip::new(state, |chip| (Box::new(legs) as _, chip));
-            let binding = Binding2::new(state, |(_, in_1), (_, in_2)| Some(in_1 | in_2));
+            let binding = Binding2::new(state, (), |(), (_, in_1), (_, in_2)| Some(in_1 | in_2));
             binding.set_source_1(state, &mut OrLegs::IN_1.source(chip.legs()));
             binding.set_source_2(state, &mut OrLegs::IN_2.source(chip.legs()));
             OrLegs::OUT.bind_distinct(state, chip.legs(), binding);
@@ -118,7 +118,7 @@ mod not_chip {
         pub fn new(state: &mut dyn State) -> Chip {
             let legs = Self::new_priv();
             let chip = Chip::new(state, |chip| (Box::new(legs) as _, chip));
-            let binding = Binding1::new(state, |(_, in_1): (bool, bool)| Some(!in_1));
+            let binding = Binding1::new(state, (), |(), (_, in_1): (bool, bool)| Some(!in_1));
             binding.set_source_1(state, &mut NotLegs::IN_.source(chip.legs()));
             NotLegs::OUT.bind_distinct(state, chip.legs(), binding);
             chip
@@ -190,20 +190,20 @@ fn main() {
         chips: chips.clone(),
     };
 
-    let not_1_out_to_or_2_in = Binding1::new(state, |(_, value)| Some(value));
+    let not_1_out_to_or_2_in = Binding1::new(state, (), |(), (_, value)| Some(value));
     not_1_out_to_or_2_in.set_source_1(state, &mut NotLegs::OUT.source(chips.not_1.legs()));
     OrLegs::IN_2.bind_uncond(state, chips.or_2.legs(), not_1_out_to_or_2_in);
-    let not_2_out_to_or_1_in = Binding1::new(state, |(_, value)| Some(value));
+    let not_2_out_to_or_1_in = Binding1::new(state, (), |(), (_, value)| Some(value));
     not_2_out_to_or_1_in.set_source_1(state, &mut NotLegs::OUT.source(chips.not_2.legs()));
     OrLegs::IN_2.bind_uncond(state, chips.or_1.legs(), not_2_out_to_or_1_in);
-    let or_1_out_to_not_1_in = Binding1::new(state, |(_, value)| Some(value));
+    let or_1_out_to_not_1_in = Binding1::new(state, (), |(), (_, value)| Some(value));
     or_1_out_to_not_1_in.set_source_1(state, &mut OrLegs::OUT.source(chips.or_1.legs()));
     NotLegs::IN_.bind_uncond(state, chips.not_1.legs(), or_1_out_to_not_1_in);
-    let or_2_out_to_not_2_in = Binding1::new(state, |(_, value)| Some(value));
+    let or_2_out_to_not_2_in = Binding1::new(state, (), |(), (_, value)| Some(value));
     or_2_out_to_not_2_in.set_source_1(state, &mut OrLegs::OUT.source(chips.or_2.legs()));
     NotLegs::IN_.bind_uncond(state, chips.not_2.legs(), or_2_out_to_not_2_in);
 
-    let print_out = Binding1::new(state, |x| Some(x));
+    let print_out = Binding1::new(state, (), |(), x| Some(x));
     print_out.set_source_1(state, &mut NotLegs::OUT.source(chips.not_2.legs()));
     print_out.set_target_fn(state, (), |_, (), (old, new)| {
         let old = if old { "1" } else { "0" };
